<?php

/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Malin Lorch <mlorch@liquidweb.com>
 * @date    Dec 17, 2020
 */

/**
 * Class for installing acronis backup cloud.
 *
 * @package InterWorx
 * @subpackage CLI
 */
class AcronisDbSeeder extends CLI
{
    const PARAM_USERNAME = 'username';
    const DB_NAME_1 = 'testdb1';
    const DB_NAME_2 = 'testdb2';
    const DB_NAME_SYSTEM = 'testdbsystem';

    const DOMAIN = 'acronistest.com';
    const UNIQNAME_DEFAULT = 'acronistest';
    const DBS = array(self::DB_NAME_2, self::DB_NAME_1);

    /**
     * Register functions, install & uninstall.
     */
    public function registerOptArgs()
    {
        $this->_func('createAllDatabases', 'create all databases - sw & system');
        $this->_func('dropAllDatabases', 'drop all databases - sw & system');
        $this->_func('createAccountAndDatabases', 'create sw account & sw databases & system db');
        $this->_func('createSw', 'create sw account');
        $this->_func('deleteSw', 'delete sw account');
        $this->_func('createSwDatabases', 'create & seed sw dbs');
        $this->_func('dropSwDatabases', 'drop sw dbs');
        $this->_func('createSystemDatabase', 'create system db - not connected with sw');
        $this->_func('dropSystemDatabase', 'drop system database');

        $this->_param(self::PARAM_USERNAME . '=s', 'SW username. Defaults to ' . self::UNIQNAME_DEFAULT);
    }

    /**
     * create all databases.
     */
    protected function _createAllDatabases()
    {
        $username = $this->_getUsername();

        $this->_createSwDbs($username);
        $this->_createSystemDb();

        $this->success('successfully created all dbs');
    }

    /**
     * Get username. Defaults to 'acronistest' if no option is defined
     */
    private function _getUsername()
    {
        $username = self::UNIQNAME_DEFAULT;

        if ($this->_isOptionDefined(self::PARAM_USERNAME)) {
            $username = $this->_getOption(self::PARAM_USERNAME);
        }

        return $username;
    }

    /**
     * drop all databases.
     */
    protected function _dropAllDatabases()
    {
        $this->_dropSwDbs();
        $this->_dropSystemDb();

        $this->success('successfully dropped all dbs');
    }

    /**
     * create sw account, sw databases, and system database.
     */
    protected function _createAccountAndDatabases()
    {
        $this->_createSwAccount();
        $this->_createSwDbs();
        $this->_createSystemDb();

        $this->success('successfully created all accounts & dbs');
    }

    /**
     * create sw account, db, and seed db.
     */
    protected function _createSw()
    {
        $this->_createSwAccount();
        $this->success('succsessfully created sw account');
    }

    /**
     * Create and seed dbs for sw account.
     */
    protected function _createSwDatabases()
    {
        $this->_createSwDbs();
        $this->success('successfully created databases');
    }

    /**
     * create sw dbs (creates 2 test dbs under same account).
     */
    private function _createSwDbs()
    {
        $this->_validateSwAccountExists();

        foreach (self::DBS as $dbName) {
            $this->_createSwDb($dbName);
        }
    }

    /**
     * Add sw db.
     *
     * @param string $dbName
     */
    private function _createSwDb($dbName)
    {
        if (!$this->_swDbExists($dbName)) {
            $this->_addSwDb($dbName);
            $this->_seedSwDb($dbName);
        }
    }

    /**
     * @param string $dbName
     * @return boolean
     */
    private function _swDbExists($dbName)
    {
        $SW = $this->_getSw();
        $MySQL = $SW->getMySQLDriver();
        $dbs = $MySQL->getDatabaseArray();
        $prefixedDb = $this->_getPrefixedDbName($dbName);

        return in_array($prefixedDb, $dbs);
    }

    /**
     * @param string $dbName
     */
    private function _addSwDb($dbName)
    {
        $SW = $this->_getSw();

        $Reply = IW::FC()->routeFromPHP(
            'Ctrl_Siteworx_MysqlDb',
            'addCommit',
            array('name' => $dbName),
            $SW
        );

        if (!$Reply->wasSuccessful()) {
            $this->fail('unable to create db: ' . $dbName);
            IW::getLog()->error($Reply->getErrors());
        }
    }

    /**
     * @param string $dbName
     */
    private function _seedSwDb($dbName)
    {
        $prefixedName = $this->_getPrefixedDbName($dbName);
        $tableName = $this->_getTableName($dbName);
        $this->_seed($prefixedName, $tableName);
    }

    /**
     * @param string $dbName
     * @return string
     */
    private function _getTableName($dbName)
    {
        return "swtable_$dbName";
    }

    /**
     * Get prefixes db name.
     *
     * @param string $name
     * @return string
     */
    private function _getPrefixedDbName($name)
    {
        $username = $this->_getUsername();
        return "{$username}_{$name}";
    }

    /**
     * create system database.
     */
    protected function _createSystemDatabase()
    {
        $this->_createSystemDb();
        $this->success('successfully created system database');
    }

    /**
     * create system db.
     */
    private function _createSystemDb()
    {
        $this->_createSystem();
        $this->_seed(self::DB_NAME_SYSTEM, 'systemdb');
    }

    /**
     * drop system db
     */
    private function _dropSystemDb()
    {
        $sql = "DROP DATABASE " . self::DB_NAME_SYSTEM;
        $dsn = Ini::get(Ini::MYSQL, 'rootdsn');
        $db = IWorxDbUtil::db_connect($dsn, false);

        IWorxDbUtil::db_query($db, $sql);
        $db->disconnect();
    }

    /**
     * drop system database.
     */
    protected function _dropSystemDatabase()
    {
        try {
            $this->_dropSystemDb();
        } catch (Exception $e) {
            if (strpos($e->getMessage(), "database doesn't exist") !== false) {
                throw new Exception_Acronis_DbNotFound(self::DB_NAME_SYSTEM);
            }
        }
    }

    /**
     * create system db
     */
    private function _createSystem()
    {
        $sql = "CREATE DATABASE IF NOT EXISTS " . self::DB_NAME_SYSTEM;
        $dsn = Ini::get(Ini::MYSQL, 'rootdsn');
        $db = IWorxDbUtil::db_connect($dsn, false);

        IWorxDbUtil::db_query($db, $sql);
        $db->disconnect();
    }

    /**
     * Seed db.
     *
     * @param string $db
     * @param string $tableName
     */
    private function _seed($db, $tableName)
    {
        $dsn = substr(Ini::get(Ini::MYSQL, 'rootdsn'), 0, -5) . $db;
        $db = IWorxDbUtil::db_connect($dsn, false);

        $sqlCreateTable = "CREATE TABLE IF NOT EXISTS `$tableName` (`id` int(11));";
        $sqlPopulateTable = "INSERT INTO `$tableName` (`id`) VALUES (1), (2), (3);";

        IWorxDbUtil::db_query($db, $sqlCreateTable);
        IWorxDbUtil::db_query($db, $sqlPopulateTable);

        $db->disconnect();
    }

    /**
     * create sw account
     */
    private function _createSwAccount()
    {
        $this->_validateSwAccountDoesNotExist();

        $email = $this->_getEmail();
        $domain = $this->_getDomain();
        $uniqname = $this->_getUsername();
        $password  = IWorxUtil::generatePassword();
        $NW = new NodeWorx(NodeWorx::MASTER_ID);
        $DS = new DataSource_NW_IP_FreeIPv4($NW);
        $ipv4 = $DS->getFirstValue();

        $acct = array(
            'master_domain' => $domain,
            'master_domain_ipv4' => $ipv4,
            'uniqname' => $uniqname,
            'email' => $email,
            'password' => $password,
            'confirm_password' => $password,
            'OPT_STORAGE' => '999999999',
            'OPT_BANDWIDTH' => '999999999',
            'OPT_EMAIL_ALIASES' => '999999999',
            'OPT_EMAIL_AUTORESPONDERS' => '999999999',
            'OPT_EMAIL_BOXES' => '999999999',
            'OPT_EMAIL_GROUPS' => '999999999',
            'OPT_FTP_ACCOUNTS' => '999999999',
            'OPT_MYSQL_DBS' => '999999999',
            'OPT_MYSQL_DB_USERS' => '999999999',
            'OPT_POINTER_DOMAINS' => '999999999',
            'OPT_SLAVE_DOMAINS' => '999999999',
            'OPT_SUBDOMAINS' => '999999999'
        );

        $this->_sendReqSwCtrl($acct, 'addCommit');
    }


    /**
     * @param array  $args
     * @param string $action
     */
    private function _sendReqSwCtrl(array $args, $action)
    {
        $NW = new NodeWorx(NodeWorx::MASTER_ID);

        $Reply = IW::FC()->routeFromPHP(
            'Ctrl_Nodeworx_Siteworx',
            $action,
            $args,
            $NW
        );

        if (!$Reply->wasSuccessful()) {
            IW::getLog()->error($Reply->getErrors());
            $this->fail($Reply->getErrorsAsString());
        }
    }

    /**
     * Delete sw account.
     */
    protected function _deleteSw()
    {
        $domain = $this->_getDomain();
        $args = array('domain' => $domain);
        $this->_sendReqSwCtrl($args, 'deleteCommit');
    }

    /**
     * Fail if siteworx account already exists.
     */
    private function _validateSwAccountDoesNotExist()
    {
        $SW = $this->_getSw();

        if ($SW->exists()) {
            $this->fail('Siteworx account already exists');
        }
    }

    /**
     * Make sure that sw account exists.
     */
    private function _validateSwAccountExists()
    {
        $SW = $this->_getSw();

        if (!$SW->exists()) {
            $this->fail('Siteworx account does not exists');
        }
    }

    /**
     * Get prefixed sw db names.
     *
     * @return array
     */
    private function _getPrefixedDbNamesSw()
    {
        return array_map(function ($dbName) {
            return $this->_getPrefixedDbName($dbName);
        }, self::DBS);
    }

    /**
     * @return Siteworx
     */
    private function _getSw()
    {
        $domain = $this->_getDomain();
        return new SiteWorx($domain);
    }

    /**
     * Drop SiteWorx databases.
     */
    protected function _dropSwDatabases()
    {
        $this->_dropSwDbs();
        $this->success('successfully dropped sw databases');
    }

    /**
     * drop SW databases.
     */
    private function _dropSwDbs()
    {
        $this->_validateSwAccountExists();

        foreach ($this->_getPrefixedDbNamesSw() as $dbName) {
            $this->_deleteSwDb($dbName);
        }
    }

    /**
     * Delete sw db.
     *
     * @param string $dbName
     */
    private function _deleteSwDb($dbName)
    {
        $SW = $this->_getSw();

        $Reply = IW::FC()->routeFromPHP(
            'Ctrl_Siteworx_MysqlDb',
            'deleteCommit',
            array('name' => $dbName),
            $SW
        );

        if (!$Reply->wasSuccessful()) {
            IW::getLog()->error($Reply->getErrors());
        }
    }

    /**
     * Get sw domain.
     *
     * @return string
     */
    private function _getDomain()
    {
        $name = $this->_getUsername();
        return "$name.com";
    }

    /**
     * Get sw email.
     *
     * @return string
     */
    private function _getEmail()
    {
        $name = $this->_getUsername();
        return "$name@$name.com";
    }
}
