<?php

/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Malin Lorch <mlorch@liquidweb.com>
 * @date    Jun 12, 2020
 */

/**
 *
 * @package Interworx
 * @subpackage Plugin_Acronis
 */
class Ctrl_Siteworx_Acronis extends Ctrl_Siteworx_Plugin
{
    const IS_AUTHORIZED = 'authorized';
    const CONTACT_ADMIN = 'display_contact_admin';

    const PLAN_ID = 'planId';
    const MESSAGE = 'message';
    const DIRECTORIES = 'directories';
    const RECOVERY_ID = 'recoveryId';
    const RECOVERY_ERROR = 'recoveryError';
    const UNIXUSER = 'unixuser';
    const DATABASES = 'databases';
    const RESTORE_ID = 'restoreId';
    const CODE_ERROR = 'codeError';
    const CODE_TYPE = 'codeType';
    const STATE = 'state';
    const PROGRESS = 'progress';
    const LOCATION = 'location';
    const SUBTOTAL = 'subtotal';
    const UNIT = 'unit';

    private $_ActivitesLogger;
    private $_Plugin;

    /**
     * Allow actions if master user.
     *
     * @param string $action
     */
    public function isActionAllowed($action)
    {
        if (IW::SW()->getWorkingUser()->isMaster()) {
            return true;
        }

        return false;
    }

    /**
     * Load backups.
     */
    public function indexAction()
    {
        $isAuthorized = $this->_getAcronisPlugin()->userIsAuthorized();

        if (!$isAuthorized) {
            $this->_displayFailure('_notAuthorized', AcronisReply::LANG_NOT_AUTHORIZED, AcronisReply::ERROR_API_INVALID_CREDENTIALS);
        }

        $BackupAgent = $this->_getAcronisPlugin()->getBackupAgent();
        $backupAgentNotInstalled = !$BackupAgent->isInstalled();
        $scriptsMissing = $BackupAgent->scriptsMissing();

        $BackupPlan = new Model_Backups_BackupPlan();
        $isProtected = $BackupPlan->isProtected();

        $version = Ini::get(Ini::IWORX, 'rpm.release');
        $major_version = substr($version, 0, 1);
        $this->getView()->assign('iw_major_version', $major_version);

        if ($backupAgentNotInstalled || $scriptsMissing || !$isProtected) {
            $View = $this->getView();
            $View->assign(self::CONTACT_ADMIN, true);
        }

        // js handles what to display by xhr requests if installed & authorized
    }

    /**
     * Exists because index action can't refer to itself
     * when displayFailure is triggered.
     */
    private function _notAuthorizedAction()
    {
        $View = $this->getView();
        $View->assign(self::IS_AUTHORIZED, false);
    }

    /**
     * Get backups.
     *
     * @param Form_SubmitOnly $Form
     */
    public function listBackupsCommitAction(Form_SubmitOnly $Form)
    {
        $this->_executeFunctionAndCatchException('_listBackups', $Form);
    }

    /**
     * Get backups.
     *
     * @param Form_SubmitOnly $Form
     */
    private function _listBackups(Form_SubmitOnly $Form)
    {
        $Plugin = $this->_getAcronisPlugin();
        $BackupsList = $Plugin->getSwBackupsList();
        $backups = $BackupsList->getBackups(IW::SW()->getUniqname());
        $this->_displayPayloadAsJson($backups);
    }

    /**
     * @return Plugin_Acronis
     */
    private function _getAcronisPlugin()
    {
        if (!isset($this->_Plugin)) {
            $this->_Plugin = $this->_getPlugin();
        }

        return $this->_Plugin;
    }

    /**
     * Get root content.
     *
     * @param Form_AcronisBackups $Form
     */
    public function getRootContentCommitAction(Form_AcronisBackups $Form)
    {
        $this->_executeFunctionAndCatchException('_getRootContent', $Form);
    }

    /**
     * Get root content.
     *
     * @param Form_AcronisBackups $Form
     */
    private function _getRootContent(Form_AcronisBackups $Form)
    {
        try {
            $backupId = $Form->getBackupId();
            $FileRecoveryPoint = $this->_getAcronisPlugin()->getSwFilesRecoveryPoint($backupId);
            $dirContent = $FileRecoveryPoint->getRootContent();
            $this->_displayPayloadAsJson($dirContent);
        } catch (Exception_Acronis_InvalidPath $e) {
            throw new Exception_Acronis_AccountDoesNotExist(IW::SW()->getUniqname());
        }
    }

    /**
     * Get directory content by id.
     *
     * @param Form_AcronisBackups_ListDirectoryContent $Form
     */
    public function getDirectoryContentCommitAction(Form_AcronisBackups_ListDirectoryContent $Form)
    {
        $this->_executeFunctionAndCatchException('_getDirectoryContent', $Form);
    }


    /**
     * Get directory content by id.
     *
     * @param Form_AcronisBackups_ListDirectoryContent $Form
     */
    private function _getDirectoryContent(Form_AcronisBackups_ListDirectoryContent $Form)
    {
        $backupId = $Form->getBackupId();
        $path = $Form->getDirectoryPath();
        $dirContent = $this->_getDirContent($backupId, $path);
        $this->_displayPayloadAsJson($dirContent);
    }

    /**
     * @param string $backupId
     * @param string $path
     * @return array
     */
    private function _getDirContent($backupId, $path)
    {
        $FileRecoveryPoint = $this->_getAcronisPlugin()->getSwFilesRecoveryPoint($backupId);
        return $FileRecoveryPoint->getDirectoryContent($path);
    }

    /**
     * Recover directories.
     *
     * @param Form_AcronisBackups_RecoverDirectories $Form
     */
    public function recoverDirectoriesCommitAction(Form_AcronisBackups_RecoverDirectories $Form)
    {
        $this->_executeFunctionAndCatchException('_recoverDirectories', $Form);
    }

    /**
     * Recover directories.
     *
     * @param Form_AcronisBackups_RecoverDirectories $Form
     */
    private function _recoverDirectories(Form_AcronisBackups_RecoverDirectories $Form)
    {
        $this->_validateNoRestoresInitiated();
        $this->_validateNoActiveFileRestores();
        $this->_validateNoActiveDatabaseRestores();

        $backupId = $Form->getBackupId();
        $dirPaths = $Form->getDirectoryPaths();
        $shouldDeleteNewFiles = $Form->deleteRequested();

        $result = $this->_restoreDirs($backupId, $dirPaths, $shouldDeleteNewFiles);
        $this->_displayPayloadAsJson($result);
    }

    /**
     * Restore directories.
     *
     * @param string  $backupId
     * @param array   $dirs
     * @param boolean $shouldDeleteNewFiles
     */
    private function _restoreDirs($backupId, array $dirs, $shouldDeleteNewFiles)
    {
        $result = array(
            self::MESSAGE => 'No files to restore',
            self::PLAN_ID => '',
            self::DIRECTORIES => $dirs
        );

        if (!empty($dirs)) {
            $result = $this->_restoreDirsAndHandleInitiated($backupId, $dirs, $shouldDeleteNewFiles);
        }

        return $result;
    }

    /**
     * set restore initiated before attempt and
     * clear restore initiated when done/ if failure
     *
     * @param string  $backupId
     * @param array   $dirs
     * @param boolean $shouldDeleteNewFiles
     */
    private function _restoreDirsAndHandleInitiated($backupId, array $dirs, $shouldDeleteNewFiles)
    {
        try {
            Model_Recovery_Files::setRestoreInitiated();
            $result = $this->_restoreDirsAwaitApi($backupId, $dirs, $shouldDeleteNewFiles);
            Model_Recovery_Files::clearRestoreInitiated();

            return $result;
        } catch (Exception $e) {
            Model_Recovery_Files::clearRestoreInitiated();
            throw $e;
        }
    }

    /**
     * Restore directories and await acronis api update.
     * Also adds to sw file recoveries cache.
     *
     * @param string  $backupId
     * @param array   $dirs
     * @param boolean $shouldDeleteNewFiles
     */
    private function _restoreDirsAwaitApi($backupId, array $dirs, $shouldDeleteNewFiles)
    {
        $FileRecoveryPoint = $this->_getAcronisPlugin()->getSwFilesRecoveryPoint($backupId);
        $FileRecoveryPoint->recoverDirectories($dirs, $shouldDeleteNewFiles);

        $planId = $FileRecoveryPoint->getPlanId();
        $message = AcronisReply::LANG_IN_PROGRESS;

        $this->_getActivitiesLogger()->logRestoreFiles($dirs);
        $this->_awaitRecoveryApiUpdate($planId);
        $this->_addDirRecoveryToCache($planId, $dirs);

        return array(
            self::MESSAGE => $message,
            self::PLAN_ID => $planId,
            self::DIRECTORIES => $dirs
        );
    }

    /**
     * Await acronis api to update - including plan id.
     *
     * @param string $planId
     */
    private function _awaitRecoveryApiUpdate($planId)
    {
        $attempts = 0;
        while (!$this->_planExistsInApi($planId) && $attempts < 20) {
            $attempts++;
            sleep(3);
        }
    }

    /**
     * Check if recovery plan exists in acronis api.
     *
     * @param string $planId
     */
    private function _planExistsInApi($planId)
    {
        $RecoveryPlans = new Model_RecoveryPlan_Plans();
        return $RecoveryPlans->planExists($planId);
    }

    /**
     * Check recovery state & progress by recovery plan id.
     *
     * @param Form_AcronisBackups_RecoveryProgress $Form
     */
    public function getRecoveryProgressAction(Form_AcronisBackups_RecoveryProgress $Form)
    {
        $this->_executeFunctionAndCatchException('_getRecoveryProgress', $Form);
    }

    /**
     * Check recovery state by recovery plan id.
     *
     * @param Form_AcronisBackups_RecoveryProgress $Form
     */
    private function _getRecoveryProgress(Form_AcronisBackups_RecoveryProgress $Form)
    {
        $planId = $Form->getPlanId();

        $RecoveryPlans = new Model_RecoveryPlan_Plans();
        $RecoveryPlan = $RecoveryPlans->getRecoveryPlanById($planId);

        $result = array(
            self::PROGRESS => $RecoveryPlan->getProgress(),
            self::STATE => $RecoveryPlan->getState(),
            self::CODE_TYPE => $RecoveryPlan->getCodeType(),
            self::CODE_ERROR => $RecoveryPlan->getCodeError(),
            self::RECOVERY_ID => $RecoveryPlan->getUuid(),
            self::PLAN_ID => $RecoveryPlan->getPlanId(),
            self::RECOVERY_ERROR => $RecoveryPlan->getError()
        );

        $this->_displayPayloadAsJson($result);
    }


    /**
     * Resume recovery if state is "paused".
     *
     * @param Form_AcronisBackups_ResumeRecovery $Form
     */
    public function resumeRecoveryAction(Form_AcronisBackups_ResumeRecovery $Form)
    {
        $this->_executeFunctionAndCatchException('_resumeRecovery', $Form);
    }

    /**
     * Resume recovery if state is "paused".
     *
     * @param Form_AcronisBackups_ResumeRecovery $Form
     */
    private function _resumeRecovery(Form_AcronisBackups_ResumeRecovery $Form)
    {
        $recoveryId = $Form->getRecoveryId();
        $RecoveryPlans = new Model_RecoveryPlan_Plans();
        $RecoveryPlans->resumeRecovery($recoveryId);

        $this->_displayPayloadAsJson(array(
            self::MESSAGE => 'recovery resumed'
        ));
    }


    /**
     * Get restore location.
     */
    public function getDefaultRestoreLocationAction()
    {
        $altLocation = Model_Recovery_Files_Siteworx::getDefaultRestoreLocation(IW::SW());
        $Payload = new Payload(array(
            self::LOCATION => $altLocation
        ));

        $this->_displayPayload($Payload);
    }



    /**
     * Restore directories to alt location.
     *
     * @param Form_AcronisBackups_RestoreAltLocation $Form
     */
    public function restoreToAltLocationCommitAction(Form_AcronisBackups_RestoreAltLocation $Form)
    {
        $this->_executeFunctionAndCatchException('_restoreToAltLocation', $Form);
    }

    /**
     * Restore directories to alt location.
     *
     * @param Form_AcronisBackups_RestoreAltLocation $Form
     */
    private function _restoreToAltLocation(Form_AcronisBackups_RestoreAltLocation $Form)
    {
        $this->_validateNoRestoresInitiated();
        $this->_validateNoActiveFileRestores();

        $backupId = $Form->getBackupId();
        $dirPaths = $Form->getDirectoryPaths();

        $this->_restoreToAltLocationAndHandleInitiated($backupId, $dirPaths);
    }

    /**
     * @param string $backupId
     * @param array  $dirPaths
     */
    private function _restoreToAltLocationAndHandleInitiated($backupId, array $dirPaths)
    {
        try {
            Model_Recovery_Files::setRestoreInitiated();
            $result = $this->_restoreToAltLocationAwaitApi($backupId, $dirPaths);
            Model_Recovery_Files::clearRestoreInitiated();

            $this->_displayPayloadAsJson($result);
        } catch (Exception $e) {
            Model_Recovery_Files::clearRestoreInitiated();
            throw $e;
        }
    }

    /**
     * @param string $backupId
     * @param array  $dirPaths
     */
    private function _restoreToAltLocationAwaitApi($backupId, array $dirPaths)
    {
        $FileRecoveryPoint = $this->_getAcronisPlugin()->getSwFilesRecoveryPoint($backupId);
        $FileRecoveryPoint->restoreToAltLocation($dirPaths);

        $planId = $FileRecoveryPoint->getPlanId();

        $this->_getActivitiesLogger()->logDownloadFiles($dirPaths);
        $this->_awaitRecoveryApiUpdate($planId);
        $this->_addDirRecoveryToCache($planId, $dirPaths);

        $altLocation = Model_Recovery_Files_Siteworx::getDefaultRestoreLocation(IW::SW());

        return array(
            self::MESSAGE => AcronisReply::LANG_IN_PROGRESS_ALT_LOCATION . " $altLocation",
            self::PLAN_ID => $planId,
            self::DIRECTORIES => $dirPaths
        );
    }

    /**
     * Add recovery to cache.
     *
     * @param string $planId
     * @param array  $dirPaths
     */
    private function _addDirRecoveryToCache($planId, array $dirPaths)
    {
        $Cache = new Model_Cache_Restore_Files();
        $unixuser = IW::SW()->getUniqname();

        $restore = $Cache::getCacheFormat($planId, $dirPaths, $unixuser);
        $Cache->add($planId, $restore);
    }

    /**
     * List databases.
     *
     * @param Form_AcronisBackups $Form
     */
    public function listDatabasesCommitAction(Form_AcronisBackups $Form)
    {
        $this->_executeFunctionAndCatchException('_listDatabases', $Form);
    }

    /**
     * List databases.
     *
     * @param Form_AcronisBackups $Form
     */
    private function _listDatabases(Form_AcronisBackups $Form)
    {
        $backupId = $Form->getBackupId();
        $unixUser = IW::SW()->getUniqname();
        $DbRestorer = $this->_getAcronisPlugin()->getSwDatabaseRecoveryPoint($backupId, $unixUser);
        $dbs = $DbRestorer->getDbs();

        $this->_displayPayloadAsJson($dbs);
    }

    /**
     * Restore datbases.
     * @param Form_AcronisBackups_Siteworx_RecoverDatabases $Form
     */
    public function restoreDatabasesCommitAction(Form_AcronisBackups_Siteworx_RecoverDatabases $Form)
    {
        $this->_executeFunctionAndCatchException('_restoreDatabases', $Form);
    }

    /**
     * Restore databases.
     *
     * @param Form_AcronisBackups_Siteworx_RecoverDatabases $Form
     */
    private function _restoreDatabases(Form_AcronisBackups_Siteworx_RecoverDatabases $Form)
    {
        $backupId = $Form->getBackupId();
        $databases = $Form->getDatabases();

        $this->_validateDbsExist($backupId, $databases);
        $this->_validateNoRestoresInitiated();
        $this->_validateNoActiveDatabaseRestores();
        $this->_validateNoActiveFileRestores();

        $payloadData = $this->_restoreDbs($backupId, $databases);

        $this->_displayPayloadAsJson($payloadData);
    }

    /**
     * @param string $backupId
     * @param array  $dbs
     */
    private function _validateDbsExist($backupId, array $dbs)
    {
        $unixuser = IW::SW()->getUniqname();
        $DbRestorer = $this->_getAcronisPlugin()->getSwDatabaseRecoveryPoint($backupId, $unixuser);
        $userdbs = $DbRestorer->getDbs();

        foreach ($dbs as $dbName) {
            if (!in_array($dbName, $userdbs)) {
                throw new Exception_Acronis_DbNotFound($dbName);
            }
        }
    }


    /**
     * Throws exception if user has active restores.
     *
     * @throws Exception
     */
    private function _validateNoActiveFileRestores()
    {
        $RecoveryPlans = new Model_RecoveryPlan_Plans();
        $plans = $RecoveryPlans->getAll();

        foreach ($plans as $plan) {
            if ($this->_fileRestoreInProgress($plan)) {
                throw new Exception_Acronis_RestoreInProgress();
            }
        }
    }

    /**
     * Check if plan is in state started or enqueued.
     *
     * @param array $plan
     * @return boolean
     */
    private function _fileRestoreInProgress(array $plan)
    {
        $state = $plan[self::STATE];
        return $state === 'started' || $state === 'enqueued';
    }

    /**
     * Make sure no active db restores in cache.
     */
    private function _validateNoActiveDatabaseRestores()
    {
        $Cache = new Model_Cache_Restore_Databases();

        if ($Cache->restoresInProgress()) {
            throw new Exception_Acronis_DbRecoveryInProgress();
        }
    }

    /**
     * @param string $backupId
     * @param array  $databases
     * @param string $unixUser
     * @param string $id
     */
    private function _recoverDbsInBackground($backupId, array $databases, $unixUser, $id)
    {
        $phpbin = INI::get(INI::IWORX_BIN, 'php');
        $dbRestorerBin = INI::get(INI::IWORX_PLUGINS, 'dir') . '/acronis/bin/acronisdbrestorer.php';
        $functionToCall = '--restoreSiteWorxDatabases';

        $dbs = implode(",", $databases);
        $dbsArg = "--databases='$dbs'";
        $backupIdArg = "--backup_id='$backupId'";
        $userArg = "--unixuser='$unixUser'";
        $idArg = "--restore_id='$id'";
        $cmd = "$phpbin $dbRestorerBin $functionToCall $dbsArg $userArg $idArg $backupIdArg > /dev/null 2>&1 &";

        exec($cmd);
        IW::getLog()->info("Acronis: Running db restore script in background: $dbRestorerBin");
    }

    /**
     * Get database recovery status.
     *
     * @param Form_AcronisBackups_DatabaseRecoveryState $Form
     */
    public function getDatabaseRecoveryStateAction(Form_AcronisBackups_DatabaseRecoveryState $Form)
    {
        $this->_executeFunctionAndCatchException('_getDatabaseRecoveryState', $Form);
    }

    /**
     * @param Form_AcronisBackups_DatabaseRecoveryState $Form
     */
    private function _getDatabaseRecoveryState(Form_AcronisBackups_DatabaseRecoveryState $Form)
    {
        $id = $Form->getId();

        $Cache = new Model_Cache_Restore_Databases();
        $restore = $Cache->get($id);

        if (is_null($restore)) {
            throw new Exception_Acronis_InvalidInput($id);
        }

        $this->_displayPayloadAsJson($restore);
    }

    /**
     * List database recoveries
     */
    public function listDatabaseRecoveriesAction()
    {
        $unixUser = IW::SW()->getUniqname();
        $Cache = new Model_Cache_Restore_Databases();
        $restores = $Cache->getRestoresByUnixUser($unixUser);
        $this->_displayPayloadAsJson($restores);
    }

    /**
     * Restore db dumps to alt location.
     *
     * @param Form_AcronisBackups_Siteworx_RecoverDatabases $Form
     */
    public function restoreDatabasesAltLocationCommitAction(Form_AcronisBackups_Siteworx_RecoverDatabases $Form)
    {
        $this->_executeFunctionAndCatchException('_restoreDatabasesAltLocation', $Form);
    }

    /**
     * Restore db dumps to alt location.
     *
     * @param Form_AcronisBackups_Siteworx_RecoverDatabases $Form
     */
    private function _restoreDatabasesAltLocation(Form_AcronisBackups_Siteworx_RecoverDatabases $Form)
    {
        $backupId = $Form->getBackupId();
        $databases = $Form->getDatabases();

        $this->_validateDbsExist($backupId, $databases);
        $this->_validateNoRestoresInitiated();
        $this->_validateNoActiveFileRestores();
        $this->_validateNoActiveDatabaseRestores();

        $this->_restoreDbsAltLocationHandleInitiated($backupId, $databases);
    }

    /**
     * Restore databases alt location & set restore initiated.
     * Clears initiated when done/if failure.
     *
     * @param string $backupId
     * @param array  $databases
     */
    private function _restoreDbsAltLocationHandleInitiated($backupId, array $databases)
    {
        try {
            Model_Recovery_Files::setRestoreInitiated();
            $result = $this->_restoreDbsAltLocation($backupId, $databases);
            Model_Recovery_Files::clearRestoreInitiated();

            $this->_displayPayloadAsJson($result);
        } catch (Exception $e) {
            Model_Recovery_Files::clearRestoreInitiated();
            throw $e;
        }
    }

    /**
     * Restore databases alt location.
     *
     * @param string $backupId
     * @param array  $databases
     */
    private function _restoreDbsAltLocation($backupId, array $databases)
    {
        $unixUser = IW::SW()->getUniqname();

        $DbRestorer = $this->_getAcronisPlugin()->getSwDatabaseRecoveryPoint($backupId, $unixUser);
        $DbRestorer->restoreDatabasesAltLocation($databases);
        $fullAltLocation = $DbRestorer->getRestoreLocation();
        $planId = $DbRestorer->getPlanId();

        $this->_getActivitiesLogger()->logDownloadDatabases($databases);

        return array(
            self::MESSAGE => 'Database dumps successfully recovered',
            self::UNIXUSER => $unixUser,
            self::DATABASES => $databases,
            self::LOCATION => $fullAltLocation,
            self::PLAN_ID => $planId
        );
    }


    /**
     * List recoveries. Only includes recoveries triggered using InterWorx.
     *
     * @param Form_SubmitOnly $Form
     */
    public function listRecoveriesCommitAction(Form_SubmitOnly $Form)
    {
        $this->_executeFunctionAndCatchException('_listRecoveries', $Form);
    }

    /**
     * @param Form_SubmitOnly $Form
     */
    private function _listRecoveries(Form_SubmitOnly $Form)
    {
        $unixuser = IW::SW()->getUniqname();
        $RecoveryPlans = new Model_RecoveryPlan_Plans();
        $plans = $RecoveryPlans->getByUnixUser($unixuser);
        $this->_displayPayloadAsJson($plans);
    }


    /**
     * Restore databases and directories.
     *
     * @param Form_AcronisBackups_Siteworx_RecoverDatabasesAndDirectories $Form
     */
    public function restoreDatabasesAndDirectoriesCommitAction(Form_AcronisBackups_Siteworx_RecoverDatabasesAndDirectories $Form)
    {
        $this->_executeFunctionAndCatchException('_restoreDatabasesAndDirectories', $Form);
    }

    /**
     * @param Form_AcronisBackups_Siteworx_RecoverDatabasesAndDirectories $Form
     */
    private function _restoreDatabasesAndDirectories(Form_AcronisBackups_Siteworx_RecoverDatabasesAndDirectories $Form)
    {
        $backupId = $Form->getBackupId();
        $databases = $Form->getDatabases();
        $directories = $Form->getDirectories();

        $this->_validateArgsNotEmpty($databases, $directories);
        $this->_validateDbsExist($backupId, $databases);
        $this->_validateNoRestoresInitiated();
        $this->_validateNoActiveFileRestores();
        $this->_validateNoActiveDatabaseRestores();

        $shouldDeleteNewFiles = $Form->deleteRequested();
        $fileRecoveryResult = $this->_restoreDirs($backupId, $directories, $shouldDeleteNewFiles);
        $dbRecoveryResult = $this->_restoreDbs($backupId, $databases);

        $payloadData = array(
            self::DIRECTORIES => $fileRecoveryResult,
            self::DATABASES => $dbRecoveryResult
        );

        $this->_displayPayloadAsJson($payloadData);
    }

    /**
     * Throw exception if restore is initiated.
     */
    private function _validateNoRestoresInitiated()
    {
        if (Model_Recovery_Files::restoreInitiated()) {
            throw new Exception_Acronis_RestoreInProgress();
        }
    }

    /**
     * @param array $dbs
     * @param array $paths
     */
    private function _validateArgsNotEmpty(array $dbs, array $paths)
    {
        if (empty($dbs) && empty($paths)) {
            throw new Exception_Acronis_InvalidInput('databases and directories are both empty');
        }
    }

    /**
     * @param string $backupId
     * @param array  $dbs
     */
    private function _restoreDbs($backupId, array $dbs)
    {
        $message = 'No databases to restore';
        $unixuser = IW::SW()->getUniqname();
        $id = '';

        if (!empty($dbs)) {
            $id = IWorxSecUtil::getUuid();
            $message = 'Database Recovery Started';
            $this->_recoverDbsInBackground($backupId, $dbs, $unixuser, $id);
            $this->_getActivitiesLogger()->logRestoreDatabases($dbs);
        }

        return array(
            self::MESSAGE => $message,
            self::UNIXUSER => $unixuser,
            self::DATABASES => $dbs,
            self::RESTORE_ID => $id
        );
    }

    /**
     * Get restore size in B for databases and files.
     *
     * @param Form_AcronisBackups_Siteworx_GetSize $Form
     */
    public function getRestoreSizeCommitAction(Form_AcronisBackups_Siteworx_GetSize $Form)
    {
        $this->_executeFunctionAndCatchException('_getSize', $Form);
    }

    /**
     * @param Form_AcronisBackups_Siteworx_GetSize $Form
     */
    private function _getSize(Form_AcronisBackups_Siteworx_GetSize $Form)
    {
        $backupId = $Form->getBackupId();
        $unixuser = IW::SW()->getUniqname();

        $databases = $Form->getDatabases();
        $directories = $Form->getDirectories();

        if (empty($databases) && empty($directories)) {
            throw new Exception_Acronis_InvalidInput($databases);
        }

        $Backup = $this->_getBackup($backupId);
        $sizeDatabases = $Backup->getSizeDatabases($databases, $unixuser);

        $FileRecovery = $this->_getAcronisPlugin()->getSwFilesRecoveryPoint($backupId);
        $sizeDirectories = $FileRecovery->getSizeDirectories($directories);

        $payloadData = AcronisReply_Restore::getRestoreSize($sizeDatabases, $sizeDirectories);

        $this->_displayPayloadAsJson($payloadData);
    }

    /**
     * @param string $backupId
     * @return Model_Backups_Backup
     */
    private function _getBackup($backupId)
    {
        $Cache = new Model_Cache_Backup();
        $apiDetails = $Cache->getApiData($backupId);
        return new Model_Backups_Backup($apiDetails);
    }


    /**
     *
     * @param string $functionToCall
     * @param Form   $Form
     */
    private function _executeFunctionAndCatchException($functionToCall, Form $Form)
    {
        try {
            $this->$functionToCall($Form);
        } catch (Exception_Acronis $e) {
            IW::getLog()->error("Action failed $functionToCall");
            $e->logError();

            $errorCode = $e->getErrorCode();
            $langKey = AcronisReply::getErrorLangKey($errorCode);
            $this->_displayFailure('index', $langKey, $errorCode);
        } catch (Exception $e) {
            IW::getLog()->error("Action failed $functionToCall");
            IW::getLog()->warning("Acronis: " . $e->getMessage());

            $errorCode = Exception_Acronis::getDefaultErrorCode();
            $langKey = AcronisReply::getErrorLangKey($errorCode);
            $this->_displayFailure('index', $langKey, $errorCode);
        }
    }

    /**
     * Display payload as json if CLI
     *
     * @param mixed $payloadContent
     */
    private function _displayPayloadAsJson($payloadContent)
    {
        // render as json
        if (IW::Env()->matches(IW::CLI)) {
            IW::Env()->setOutputStyle('pretty');
        }
        $Payload = new Payload($payloadContent);
        $this->_displayPayload($Payload);
    }

    /**
     * get sw activities logger.
     *
     * @return Model_Logger_Activities_Siteworx
     */
    protected function _getActivitiesLogger()
    {
        if (!isset($this->_ActivitesLogger)) {
            $this->_ActivitesLogger = new Model_Logger_Activities_Siteworx();
        }

        return $this->_ActivitesLogger;
    }
}
