<?php

/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2022 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @author     Malin Lorch <mlorch@liquidweb.com>
 * @date       Jul 01, 2022
 */

class Model_Api_Private_Ams_RecoveryPlan extends Model_Api
{
    const API_ENDPOINT_PLAN_DRAFT = '/bc/api/ams/recovery/plan_drafts';
    const API_ENDPOINT_RUN_BACKUP = '/bc/api/ams/recovery/plan_operations/run';

    /**
     * recover directories to alt location.
     *
     *
     * @param string $legacyId
     * @param string $agentTenantId
     * @param array  $dirIds
     * @param string $altLocation
     */
    public function recoverDirectoriesToAltLocation($legacyId, $agentTenantId, array $dirIds, $altLocation)
    {
        $draft = $this->_createRecDraft($legacyId, $agentTenantId, $dirIds);
        $draftId = $draft['DraftID'];

        $this->_setRecoveryLocation($draftId);
        $this->_setRecoveryAltLocation($draftId, $altLocation);
        $this->_editRecDraftOptions($draftId);

        $plan = $this->_createRecoveryPlan($draftId);
        $this->_planId = $plan['PlanID'];

        $this->_runRecovery($this->_planId);
    }


    /**
     * Recover directories.
     *
     * @param Model_Backups_Backup $Backup
     * @param array                $dirIds
     * @param boolean              $deleteNewContent
     */
    public function recoverDirectories(Model_Backups_Backup $Backup, array $dirIds, $deleteNewContent = false)
    {
        $draft = $this->_createRecDraft($Backup->getLegacyId(), $Backup->getTenantAgentId(), $dirIds);
        $draftId = $draft['DraftID'];

        $this->_setRecoveryLocation($draftId);

        $deleteAfter = $deleteNewContent ? $Backup->getCreated() : null;
        $this->_editRecDraftOptions($draftId, $deleteAfter);

        $plan = $this->_createRecoveryPlan($draftId);
        $this->_planId = $plan['PlanID'];

        $this->_runRecovery($this->_planId);
    }

    /**
     * Needed to check progress and state of recovery.
     *
     * @return string
     */
    public function getPlanId()
    {
        return $this->_planId;
    }

    /**
     *
     *
     * @param string $draftId
     * @return array
     */
    private function _setRecoveryLocation($draftId)
    {
        $agentId = $this->_getAgentId();
        $endpoint = self::API_ENDPOINT_PLAN_DRAFT . "/$draftId/target_location?machineId=$agentId";
        $config = array(
            'data' => array(
                'machineId' => $agentId,
                'type' => 'original',
                'subscriptionId' => $this->_getSubscriptionId()
            )
        );

        return $this->_getCookieClient()->sendAuthenticatedPutRequest($endpoint, $config);
    }

    /**
     * set recovery alt location.
     *
     * @param string $draftId
     * @param string $altLocation
     */
    private function _setRecoveryAltLocation($draftId, $altLocation)
    {
        $endpoint = self::API_ENDPOINT_PLAN_DRAFT . "/$draftId/target_location";
        $data = array(
            'type' => 'local_folder',
            'path' => $altLocation
        );

        $config = array('data' => $data);
        $res = $this->_getCookieClient()->sendAuthenticatedPutRequest($endpoint, $config);

        return $res;
    }

    /**
     * Recover directories.
     *
     * @param string $legacyId
     * @param string $agentTenantId
     * @param array  $dirIds
     */
    private function _createRecDraft($legacyId, $agentTenantId, array $dirIds)
    {
        $machineId = $this->_getAgentId();
        $apiEndpoint = self::API_ENDPOINT_PLAN_DRAFT . "?machineId=$machineId";
        $postData = $this->_getRecoveryPostData($legacyId, $agentTenantId, $dirIds);
        return $this->_getCookieClient()->sendAuthenticatedPostRequest($apiEndpoint, array('data' => $postData));
    }

    /**
     * get recovery post data.
     *
     * @param string $legacyId
     * @param string $agentTenantId
     * @param array  $dirIds
     * @return array
     */
    private function _getRecoveryPostData($legacyId, $agentTenantId, array $dirIds)
    {
        return array(
            'backupId' => $legacyId,
            'machineId' => $this->_getAgentId(),
            'resourcesIds' => $dirIds,
            'subscriptionId' => $this->_getSubscriptionId(),
            'agentTenantId' => $agentTenantId
        );
    }

    /**
     * Edit recovery draft.
     *
     * @param string $draftId
     * @param mixed  $deleteAfter
     */
    private function _editRecDraftOptions($draftId, $deleteAfter = null)
    {
        $endpoint = self::API_ENDPOINT_PLAN_DRAFT . "/$draftId/options";
        $data = array(
            'recoverPriority' => 'normal',
            'errorHandling' => $this->_getErrorHandling(),
            'silentReboot' => false,
            'validateBackupsBeforeRecover' => false,
            'prePostCommands' => $this->_getPrePostCommands($deleteAfter),
            'fileOptions' => $this->_getFileOptions(),
            'subscriptionId' => $this->_getSubscriptionId()
        );

        return $this->_getCookieClient()->sendAuthenticatedPutRequest($endpoint, array('data' => $data));
    }


    /**
     *
     *
     * @return array
     */
    private function _getErrorHandling()
    {
        return array(
            'reattemptsCount' => 30,
            'silentMode' => false,
            'reattemptOnError' => true,
            'reattemptTimeFrame' => array(
                'value' => 30,
                'type' => 'seconds'
            )
        );
    }

    /**
     * Get pre/post commands.
     *
     * @param boolean $deleteAfter
     * @return array
     */
    private function _getPrePostCommands($deleteAfter = null)
    {
        $postCommand = $this->_getPostCommand($deleteAfter);
        $preConfig = $this->_getPrePostCommandConfig();
        $postConfig = $this->_getPrePostCommandConfig($postCommand);

        $useDefaultCommands = !isset($postCommand);
        $usePostCommands = isset($postCommand);

        return array(
            'preCommands' => $preConfig,
            'postCommands' => $postConfig,
            'usePostCommands' => $usePostCommands,
            'usePreCommands' => false,
            'useDefaultCommands' => $useDefaultCommands
        );
    }

    /**
     * Get post command - delete newer files.
     *
     * @param mixed $deleteAfter
     * @return string
     */
    private function _getPostCommand($deleteAfter = null)
    {
        $dirsToRecover = $this->_postCommandPaths;
        if (!is_null($deleteAfter) && !empty($dirsToRecover)) {
            return "find $dirsToRecover -newermt \"$deleteAfter\" -prune -exec /bin/rm -rf {} +";
        }
    }

    /**
     * Get pre/post command config.
     *
     * @param string $command
     * @return array
     */
    private function _getPrePostCommandConfig($command = null)
    {
        return array(
            'continueOnCommandError' => true,
            'command' => $command === null ? '' : $command,
            'workingDirectory' => '/',
            'commandArguments' => '',
            'waitCommandComplete' => true
        );
    }

    /**
     * Set cut off date for file deletion.
     *
     * @param string $date
     */
    public function deleteFilesCreatedAfter($date)
    {
        $this->_deleteAfter = $date;
    }

    /**
     * Add path where new files should be deleted.
     *
     * @param string $path
     */
    public function addPathDeleteAfter($path)
    {
        if (!isset($this->_postCommandPaths)) {
            $this->_postCommandPaths = '';
        }

        $this->_postCommandPaths .= "$path/* ";
    }

    /**
     *
     * @return array
     */
    private function _getFileOptions()
    {
        return array(
            'preserveSecuritySettings' => true,
            'overwriteMode' => 'overwriteExistingFiles',
            'exclusions' => array(),
            'setCurrentDateTime' => false,
            'recoverWithFullPath' => false
        );
    }

    /**
     *
     *
     * @param string $draftId
     */
    private function _createRecoveryPlan($draftId)
    {
        $endpoint = self::API_ENDPOINT_PLAN_DRAFT . "/$draftId";

        $config =  array(
            'data' => array(
                'subscriptionId' => $this->_getSubscriptionId()
            )
        );

        return $this->_getCookieClient()->sendAuthenticatedPostRequest($endpoint, $config);
    }

    /**
     * Run recovery.
     *
     * @param string $planId
     */
    private function _runRecovery($planId)
    {
        $agentId = $this->_getAgentId();
        $endpoint = self::API_ENDPOINT_RUN_BACKUP . "?machineId=$agentId";
        $data = array(
            'machineId' => $agentId,
            'planId' => $planId,
            'operationId' => 'noWait'
        );

        return $this->_getCookieClient()->sendAuthenticatedPostRequest($endpoint, array('data' => $data));
    }
}
