<?php

/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2022 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @author     Malin Lorch <mlorch@liquidweb.com>
 * @date       May 25, 2022
 */


/**
 * An application is a backup plan specific to the resource.
 *
 * It is a copy of a policy. Only one application should exist per policy
 * for a specific resource of a specific type.
 */
class Model_Api_Public_Applications extends Model_Api
{
    const API_ENDPOINT_APPLICATIONS = '/api/policy_management/v4/applications';

    const TYPE_POLICY_BACKUP_MACHINE = 'policy.backup.machine';
    const TYPE_POLICY_PROTECTION_TOTAL = 'policy.protection.total';
    const TYPE_POLICY_BACKUP_INVALID = 'policy.backup.invalid';


    const STATE = 'state';
    const CONTEXT = 'context';
    const CONTEXT_ID = 'context_id';
    const CONTEXT_IDS = 'context_ids';
    const POLICY_ID = 'policy_id';
    const ID = 'id';

    /**
     * Get all plans assigned to the resource.
     *
     * @return array
     */
    public function getApplications()
    {
        // a server may have multiple applications - the relevant plan application
        // is created from policy type: policy.backup.machine
        $query = http_build_query(array(
            'context_id' => $this->_getInstanceId(),
            'policy_type' => self::TYPE_POLICY_BACKUP_MACHINE
        ));

        $endpoint = self::API_ENDPOINT_APPLICATIONS . "?$query";
        $api_res = $this->_getOAuthClient()->sendGetRequest($endpoint);
        $plans = array();

        // Acronis call plans assigned to machines "applications"
        foreach ($api_res['items'] as $applications) {
            // queried by type, so applications should only include one item (one policy type "policy.backup.machine" per machine).
            $application = $applications[0];
            $plans[] = $application;
        }

        return $plans;
    }

    /**
     * An application is a backup plan specific to the resource.
     *
     * It is a copy of a policy. Only one application should exist per policy
     * for a specific resource of a specific type.
     *
     * @param string $policy_id_backup_machine
     * @return Model_Backups_Plan_Application
     */
    public function getApplicationByPolicyId($policy_id_backup_machine): Model_Backups_Plan_Application
    {
        $query = http_build_query(array(
            'context_id' => $this->_getInstanceId(),
            'policy_id' => $policy_id_backup_machine,
        ));

        $endpoint = self::API_ENDPOINT_APPLICATIONS . "?$query";
        $api_res = $this->_getOAuthClient()->sendGetRequest($endpoint);
        $items = $api_res['items'];

        if (empty($items)) {
            throw new Exception_Acronis_BackupPlan_ApplicationDoesNotExist('No applications found for this resource with policy id: ' . $policy_id_backup_machine);
        }

        // getting nested item - querying by type and id, so both
        // should only contain one item.
        return new Model_Backups_Plan_Application($items[0][0]);
    }

    /**
     * @param string $policy_id
     */
    public function createApplication($policy_id)
    {
        $apiEndpoint = self::API_ENDPOINT_APPLICATIONS;
        $data = array(
            self::POLICY_ID => $policy_id,
            self::CONTEXT => array('items' => array($this->_getInstanceId()))
        );

        $res = $this->_getOAuthClient()->sendPostRequest($apiEndpoint, $data);
        return $res;
    }

    /**
     * Run backup.
     *
     * @param string $backup_machine_policy_id
     * @return array
     */
    public function runBackup($backup_machine_policy_id)
    {
        $endpoint = self::API_ENDPOINT_APPLICATIONS . '/run';
        // Required data does not confirm to _getRequestData format
        $data = [
            self::POLICY_ID => $backup_machine_policy_id,
            self::CONTEXT_IDS => [$this->_getInstanceId()],
            self::STATE => 'running'
        ];
        $res = $this->_getOAuthClient()->sendPutRequest($endpoint, $data);
        return $res;
    }


    /**
     * Get data object for request.
     *
     * @param string $policy_id
     * @return array
     */
    private function _getRequestData($policy_id)
    {
        return array(
            self::CONTEXT_ID => $this->_getInstanceId(),
            self::POLICY_ID => $policy_id
        );
    }

    /**
     * Delete application.
     *
     * @param string $policy_id
     * @return void
     */
    public function revokeApplication($policy_id): void
    {
        $data = $this->_getRequestData($policy_id);
        $query = http_build_query($data);
        $endpoint = self::API_ENDPOINT_APPLICATIONS . "?$query";

        // does not return any response data
        $this->_getOAuthClient()->sendDeleteRequest($endpoint);
    }
}
