<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Malin Lorch <mlorch@liquidweb.com>
 * @date    Dec 11, 2020
 */

/**
 * @package Interworx
 * @subpackage Plugin_Acronis
 */
class Model_Archives_Directory extends Model_Acronis
{
    const API_PATH = 'fullPath';
    const PATH = 'path';
    const ID = 'id';
    const API_NAME = 'displayName';
    const NAME = 'name';
    const TYPE = 'type';
    const API_MODIFIED = 'modificationTime';
    const MODIFIED = 'modified';
    const SIZE = 'size';
    const PARENT_ID = 'parentId';
    const CONTENT = 'content';
    const LOCAL_PATH = 'localPath';

    private $_path;
    private $_name;
    private $_id;
    private $_type;
    private $_modified;
    private $_size;
    private $_content;
    private $_parentId;
    private $_apiDetails;

    /**
     * expects acronis api response - array with dir info.
     *
     * @param array $apiRes
     */
    public function __construct(array $apiRes)
    {
        $this->_apiDetails = $apiRes;
        $this->_id = $apiRes[self::ID];
        $this->_path = $apiRes[self::API_PATH];
        $this->_name = $apiRes[self::API_NAME];
        $this->_type = $apiRes[self::TYPE];
        $this->_modified = $apiRes[self::API_MODIFIED];
        $this->_size = $apiRes[self::SIZE];
        $this->_parentId = $apiRes[self::PARENT_ID];
    }

    /**
     * @return string
     */
    public function getId()
    {
        return $this->_id;
    }

    /**
     * @return string
     */
    public function getName()
    {
        return $this->_name;
    }

    /**
     * @return string
     */
    public function getType()
    {
        return $this->_type;
    }

    /**
     * @return string
     */
    public function getModified()
    {
        return self::formatDate($this->_modified);
    }

    /**
     * @return string
     */
    public function getSize()
    {
        return $this->_convertBytes($this->_size);
    }

    /**
     * @return string
     */
    public function getPath()
    {
        return $this->_path;
    }

    /**
     * @return string
     */
    public function getParentId()
    {
        return $this->_parentId;
    }

    /**
     * Get all api details.
     *
     * @return array
     */
    public function getApiDetails()
    {
        return $this->_apiDetails;
    }

    /**
     * @param string $legacyId
     * @param string $machineId
     * @return array
     */
    public function getContent($legacyId)
    {
        if (!isset($this->_content)) {
            $content = $this->_getContentFromApi($legacyId);
            $this->_content = array_map(array($this, '_formatContent'), $content);
        }

        return $this->_content;
    }

    /**
     * @param string $legacyId
     * @param string $machineId
     * @return array
     */
    private function _getContentFromApi($legacyId)
    {
        $Api = $this->getArchivesApi();
        $dirs = $Api->getDirectoryContent($legacyId, $this->_id);

        return $dirs;
    }

    /**
     * @param array $dir
     */
    private function _formatContent(array $dir)
    {
        $Dir = new Model_Archives_Directory($dir);

        return array(
            self::PATH => $Dir->getPath(),
            self::ID => $Dir->getId(),
            self::NAME => $Dir->getName(),
            self::TYPE => $Dir->getType(),
            self::MODIFIED => $Dir->getModified(),
            self::SIZE => $Dir->getSize(),
            self::PARENT_ID => $Dir->getParentId()
        );
    }

    /**
     * Get dir by name.
     *
     * @param array  $directories
     * @param string $name
     */
    public static function getDirByName(array $directories, $name)
    {
        foreach ($directories as $dir) {
            if ($dir[self::NAME] === $name) {
                return $dir;
            }
        }

        throw new Exception('Directory does not exist: ' . $name);
    }

    /**
     * Get subdir by name.
     *
     * @param string $subDirName
     * @param string $legacyId
     * @param string $machineId
     * @return Model_Archives_Directory
     */
    public function getSubDirByName($subDirName, $legacyId)
    {
        $content = $this->_getContentFromApi($legacyId);

        foreach ($content as $subDir) {
            if ($subDir[self::API_NAME] === $subDirName) {
                return new Model_Archives_Directory($subDir);
            }
        }

        throw new Exception_Acronis_InvalidPath($subDirName);
    }
}
