<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2021 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @author     Malin Lorch <mlorch@liquidweb.com>
 * @date       Aug 09, 2021
 */

/**
 * @package InterWorx
 * @subpackage
 */
class prebackuptest extends IWorx_Framework_TestCase
{
    const PATH_PHP = "/home/interworx/bin/php";
    const PRE_MYSQL_PATH="/home/.acronis_tmp/pre-acronis-mysql";
    const JSON =  self::PRE_MYSQL_PATH . "/dbinfosw.json";
    const UNMAPPED_DIR = 'unmapped_dir_name';
    const TEST_DUMPS_FOLDER = Model_Recovery_Files_Databases::DUMPS_TEMP_PATH . "/acronistest-dumps";

    const NAME = 'some-db-name';
    const SIZE = 123456;
    const UNIT = 'bytes';
    const HOST = 'localhost';
    const UNIX_NO_USER = 'unixuserr';
    const UNIX_EXISTING = 'acronistest';
    const DUMPS_PATH = '/some/path/to/dump';

    private $_prescriptTimestamp;


    /**
     * remove json if any
     */
    public function setUp()
    {
        $this->_createDbs();
    }

    /**
     * Create account "acronistest" and databases.
     */
    private function _createDbs()
    {
        $runDbsBin = self::PATH_PHP . " /home/interworx/plugins/acronis/bin/acronisdbseeder.php --createAccountAndDatabases";
        IWorxExec::exec($runDbsBin);
    }

    /**
     * function runs after each test.
     */
    public function tearDown()
    {
        $this->_removeJson();
        $this->_removePrescriptFromCache();
        $this->_deleteDumps();
    }

    /**
     * Remove prescript created by test.
     */
    private function _removePrescriptFromCache()
    {
        $Cache = new Model_Cache_Prescript();
        $Cache->delete($this->_prescriptTimestamp);
    }



    /**
     * Remove json file that bin script generates.
     */
    private function _removeJson()
    {
        if ($this->_jsonExists()) {
            $this->_deleteFile(self::JSON);
        }
    }

    /**
     * @return boolean
     */
    private function _jsonExists()
    {
        return Model_AcronisFileSys::isFile(self::JSON);
    }

    /**
     * Delete dumps
     */
    private function _deleteDumps()
    {
        if (Model_AcronisFileSys::isDir(self::TEST_DUMPS_FOLDER)) {
            $Command = new Command("rm -rf " . self::TEST_DUMPS_FOLDER);
            $Command->exec();
        }
    }

    /**
     * @param string $filePath
     */
    private function _deleteFile($filePath)
    {
        $deleteJson = "rm -rf $filePath";
        $Command = new Command($deleteJson);
        $Command->runAsRoot();
        $Command->exec();

        if (!$Command->wasSuccessful()) {
            echo "COMMAND NOT SUCCESSFUL $deleteJson";
        }
    }

    /**
     * test cache from json - should cache 1 db.
     */
    public function testCacheServerInfoShouldCacheOneDbFromJson()
    {
        $dbs = array($this->_getDb(self::UNIX_EXISTING));
        $this->_writeJson($dbs);
        $this->_runPreBackupScript();

        $prescript = $this->_getPrescriptFromCache();
        $prescriptDbs = $prescript['databases_mapped'];

        $this->assertCount(1, $prescriptDbs);
    }

    /**
     * test cache from json - should cache 3 dbs.
     */
    public function testCacheServerInfoShouldCacheThreeDbsFromJson()
    {
        $dbs = array(
            $this->_getDb(self::UNIX_EXISTING),
            $this->_getDb(self::UNIX_EXISTING),
            $this->_getDb(self::UNIX_EXISTING)
        );

        $this->_writeJson($dbs);
        $this->_runPreBackupScript();

        $prescript = $this->_getPrescriptFromCache();
        $prescriptDbs = $prescript['databases_mapped'];

        $this->assertCount(3, $prescriptDbs);
    }

    /**
     * @param string $unix
     */
    private function _getDb($unix)
    {
        return array(
            'name' => self::NAME,
            'size' => self::SIZE,
            'unit' => self::UNIT,
            'host' => self::HOST,
            'dumps_path' => self::DUMPS_PATH,
            'unixuser' => $unix
        );
    }

    /**
     * Should not cache any dbs if no json & no dumps in temp path.
     */
    public function testCacheServerInfoShouldNotCacheDbsWithoutJsonAndDumps()
    {
        $this->_runPreBackupScript();

        $prescript = $this->_getPrescriptFromCache();
        $prescriptDbs = $prescript['databases_mapped'];

        $this->assertEmpty($prescriptDbs);
    }

    /**
     * Should cache dbs from dumps in temp path.
     */
    public function testCacheServerInfoShouldCacheDbsFromDumps()
    {
        $expected = 'somedatabasename';
        $this->_createDumps($expected);
        $this->_runPreBackupScript();

        $prescript = $this->_getPrescriptFromCache();
        $prescriptDbs = $prescript['databases_mapped'];

        $this->assertCount(1, $prescriptDbs);

        $db = $prescriptDbs[0];
        $this->assertSame($expected, $db['name']);
        $this->assertSame('acronistest', $db['unixuser']);
    }

    /**
     * Create dumps for test.
     *
     * @param string $dbName
     */
    private function _createDumps($dbName)
    {
        Model_AcronisFileSys::createDir(self::TEST_DUMPS_FOLDER . "/$dbName");
    }

    /**
     * Write json file.
     *
     * @param array $dbs
     */
    private function _writeJson(array $dbs)
    {
        Model_AcronisFileSys::createDir(self::PRE_MYSQL_PATH);
        IWorx_Json::writeToFile($dbs, self::JSON);
    }

    /**
     * @return array
     */
    private function _getPrescriptFromCache()
    {
        $PrescriptCache = new Model_Cache_Prescript();
        $all = $PrescriptCache->getAll();
        $timestamp = time();

        foreach ($all as $prescript) {
            $diff = $timestamp - $prescript['timestamp'];
            $matchingPrescript = $diff === 0 || $diff === 1;
            if ($matchingPrescript) {
                $this->_prescriptTimestamp = $prescript['timestamp'];
                return $prescript;
            }
        }
    }

    /**
     * run prebackup.php
     */
    private function _runPreBackupScript()
    {
        $phpBin = "/home/interworx/bin/php";
        $preBackupBin = "/home/interworx/plugins/acronis/bin/prebackup.php";
        $cmd = "$phpBin $preBackupBin --cacheServerInfo --unmapped_dir=" . self::UNMAPPED_DIR;
        $Command = new Command($cmd);
        $Command->exec();
    }
}
